@@ -1,7 +1,7 @@
 #!/bin/bash
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.135 2003/07/22 07:19:04 carpaski Exp $
+# $Header: /home/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.143 2003/10/13 07:43:38 carpaski Exp $
 
 if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 	if [ -f ${T}/successful ]; then
@@ -46,11 +46,14 @@ fi
 # Make sure it's before everything so we don't mess aliases that follow.
 unalias -a
 
+# Unset some variables that break things.
+unset GZIP BZIP BZIP2 CDPATH
+
 # We need this next line for "die" and "assert". It expands 
 # It _must_ preceed all the calls to die and assert.
 shopt -s expand_aliases
 alias die='diefunc "$FUNCNAME" "$LINENO" "$?"'
-alias assert='_retval=$?; [ $_retval = 0 ] || diefunc "$FUNCNAME" "$LINENO" "$_retval"'
+alias assert='_pipestatus="${PIPESTATUS[*]}"; [[ "${_pipestatus// /}" -eq 0 ]] || diefunc "$FUNCNAME" "$LINENO" "$_pipestatus"'
 
 OCC="$CC"
 OCXX="$CXX"
@@ -85,7 +88,11 @@ use() {
 	local x
 	for x in ${USE}; do
 		if [ "${x}" == "${1}" ]; then
-			tty --quiet < /dev/stdout || echo "${x}"
+			if [ -r /dev/fd/1 ]; then
+				tty --quiet < /dev/stdout || echo "${x}"
+			else
+			  echo "${x}"
+			fi
 			return 0
 		fi
 	done
@@ -95,13 +102,24 @@ use() {
 has() {
 	local x
 
-	local me
-	me=$1
+	local me=$1
 	shift
 	
+	# All the TTY checks really only help out depend. Which is nice.
+	# Logging kills all this anyway. Everything becomes a pipe. --NJ
 	for x in "$@"; do
 		if [ "${x}" == "${me}" ]; then
-			tty --quiet < /dev/stdout || echo "${x}"
+			if [ -r /proc/self/fd/1 ]; then
+				tty --quiet < /proc/self/fd/1 || echo "${x}"
+			elif [ -r /dev/fd/1 ]; then
+				echo "/dev/fd/1" >&2
+				tty --quiet < /dev/fd/1 || echo "${x}"
+			elif [ -r /dev/stdout ]; then
+				echo "/dev/stdout" >&2
+				tty --quiet < /dev/stdout || echo "${x}"
+			else
+				echo "${x}"
+			fi
 			return 0
 		fi
 	done
@@ -652,6 +670,9 @@ dyn_compile() {
 	[ "${LDFLAGS-unset}"     != "unset" ] && export LDFLAGS
 	[ "${ASFLAGS-unset}"     != "unset" ] && export ASFLAGS
 
+	[ "${DISTCC_DIR-unset}"  == "unset" ] && export DISTCC_DIR="${PORT_TMPDIR}/.distcc"
+	[ ! -z "${DISTCC_DIR}" ] && addwrite "${DISTCC_DIR}"
+
 	if has noauto $FEATURES &>/dev/null && [ ! -f ${BUILDDIR}/.unpacked ]; then
 		echo
 		echo "!!! We apparently haven't unpacked... This is probably not what you"
@@ -725,7 +746,7 @@ dyn_compile() {
 dyn_package() {
 	trap "abort_package" SIGINT SIGQUIT
 	cd ${BUILDDIR}/image
-	tar cpvf - * | bzip2 -f > ../bin.tar.bz2 || die "Failed to create tarball"
+	tar cpvf - ./ | bzip2 -f > ../bin.tar.bz2 || die "Failed to create tarball"
 	cd ..
 	xpak build-info inf.xpak
 	tbz2tool join bin.tar.bz2 inf.xpak ${PF}.tbz2
@@ -772,21 +793,82 @@ dyn_install() {
 	done
 	
 	if [[ $UNSAFE > 0 ]]; then
-		die "There are unsafe files. Portage will not install them."
+		die "There are ${UNSAFE} unsafe files. Portage will not install them."
 	fi
 	
 	find ${D}/ -user  portage -print0 | $XARGS -0 -n100 chown root
 	if [ "$USERLAND" == "BSD" ]; then
 		find ${D}/ -group portage -print0 | $XARGS -0 -n100 chgrp wheel
-	else	
-		find ${D}/ -group portage -print0 | $XARGS -0 -n100 chgrp root 
+	else
+		find ${D}/ -group portage -print0 | $XARGS -0 -n100 chgrp root
 	fi
+
 	echo ">>> Completed installing into ${D}"
 	echo
 	cd ${BUILDDIR}
 	trap SIGINT SIGQUIT
 }
 
+dyn_preinst() {
+	pkg_preinst
+
+	# set IMAGE depending if this is a binary or compile merge
+	[ "${EMERGE_FROM}" == "binary" ] && IMAGE=${PKG_TMPDIR}/${PF} \
+					|| IMAGE=${D}
+
+	# remove man pages
+	if has noman $FEATURES; then
+		rm -fR ${IMAGE}/usr/share/man
+	fi
+
+	# remove info pages
+	if has noinfo $FEATURES; then
+		rm -fR ${IMAGE}/usr/share/info
+	fi
+
+	# remove docs
+	if has nodoc $FEATURES; then
+		rm -fR ${IMAGE}/usr/share/doc
+	fi
+
+	# Smart FileSystem Permissions
+	if has sfperms $FEATURES; then
+		for i in $(find ${IMAGE}/ -type f -perm -4000); do
+			ebegin ">>> SetUID: [chmod go-r] $i "
+			chmod go-r $i
+			eend $?
+		done
+		for i in $(find ${IMAGE}/ -type f -perm -2000); do
+			ebegin ">>> SetGID: [chmod o-r] $i "
+			chmod o-r $i
+			eend $?
+		done
+	fi
+
+	# SELinux file labeling (needs to always be last in dyn_preinst)
+	if use selinux; then
+		# only attempt to label if setfiles is executable
+		# and 'context' is available on selinuxfs.
+		if [ -f /selinux/context -a -x /usr/sbin/setfiles ]; then
+			echo ">>> Setting SELinux security labels"
+			if [ -f ${POLICYDIR}/file_contexts/file_contexts ]; then
+				cp -f ${POLICYDIR}/file_contexts/file_contexts ${T}
+			else
+				make -C ${POLICYDIR} FC=${T}/file_contexts ${T}/file_contexts
+			fi
+
+			addwrite /selinux/context
+			/usr/sbin/setfiles -r ${IMAGE} ${T}/file_contexts ${IMAGE} \
+				|| die "Failed to set SELinux security labels."
+		else
+			# nonfatal, since merging can happen outside a SE kernel
+			# like during a recovery situation
+			echo "!!! Unable to set SELinux security labels"
+		fi
+	fi
+	trap SIGINT SIGQUIT
+}
+
 dyn_spec() {
 	tar czf /usr/src/redhat/SOURCES/${PF}.tar.gz ${O}/${PF}.ebuild ${O}/files || die "Failed to create base rpm tarball."
 
@@ -1136,6 +1218,14 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 		[ -z "${CCACHE_SIZE}" ] && export CCACHE_SIZE="2G"
 		ccache -M ${CCACHE_SIZE} &> /dev/null
 	fi
+else
+
+killparent() {
+	trap INT
+	kill -KILL ${PORTAGE_MASTER_PID}
+}
+trap "killparent" INT
+
 fi # "$*"!="depend" && "$*"!="clean"
 
 export SANDBOX_ON="1"
@@ -1183,7 +1273,7 @@ for myarg in $*; do
 	nofetch)
 		pkg_nofetch
 		;;
-	prerm|postrm|preinst|postinst|config)
+	prerm|postrm|postinst|config)
 		export SANDBOX_ON="0"
 		if [ "$PORTAGE_DEBUG" != "1" ]; then
 			pkg_${myarg}
@@ -1212,7 +1302,7 @@ for myarg in $*; do
 		fi
 		export SANDBOX_ON="0"
 		;;
-	help|clean|setup)
+	help|clean|setup|preinst)
 		#pkg_setup needs to be out of the sandbox for tmp file creation;
 		#for example, awking and piping a file in /tmp requires a temp file to be created
 		#in /etc.  If pkg_setup is in the sandbox, both our lilo and apache ebuilds break.
