@@ -1,4 +1,7 @@
-# $Header: /home/cvsroot/gentoo-src/portage/pym/xpak.py,v 1.7 2003/03/28 05:15:16 carpaski Exp $
+#
+#
+#
+# $Header: /home/cvsroot/gentoo-src/portage/pym/xpak.py,v 1.9 2003/09/29 18:13:42 carpaski Exp $
 
 # The format for a tbz2/xpak:
 #
@@ -51,7 +54,10 @@ def xpak(rootdir,outfile=None):
 	"""(rootdir,outfile) -- creates an xpak segment of the directory 'rootdir'
 	and under the name 'outfile' if it is specified. Otherwise it returns the
 	xpak segment."""
-	origdir=os.getcwd()
+	try:
+		origdir=os.getcwd()
+	except:
+		origdir="/"
 	os.chdir(rootdir)
 	mylist=[]
 
@@ -93,18 +99,27 @@ def xsplit(infile):
 	myfile=open(infile,"r")
 	mydat=myfile.read()
 	myfile.close()
-	if mydat[0:8]!="XPAKPACK":
+	
+	splits = xsplit_mem(mydat)
+	if not splits:
 		return
-	if mydat[-8:]!="XPAKSTOP":
-		return
-	indexsize=decodeint(mydat[8:12])
-	datasize=decodeint(mydat[12:16])
+	
 	myfile=open(infile+".index","w")
-	myfile.write(mydat[16:indexsize+16])
+	myfile.write(splits[0])
 	myfile.close()
 	myfile=open(infile+".dat","w")
-	myfile.write(mydat[indexsize+16:-8])
+	myfile.write(splits[1])
 	myfile.close()
+	return
+
+def xsplit_mem(mydat):
+	if mydat[0:8]!="XPAKPACK":
+		return None
+	if mydat[-8:]!="XPAKSTOP":
+		return None
+	indexsize=decodeint(mydat[8:12])
+	datasize=decodeint(mydat[12:16])
+	return (mydat[16:indexsize+16], mydat[indexsize+16:-8])
 
 def getindex(infile):
 	"""(infile) -- grabs the index segment from the infile and returns it."""
@@ -135,10 +150,10 @@ def getboth(infile):
 
 def listindex(myindex):
 	"""Print to the terminal the filenames listed in the indexglob passed in."""
-	for x in getindex(myindex):
+	for x in getindex_mem(myindex):
 		print x
 
-def getindex(myindex):
+def getindex_mem(myindex):
 	"""Returns the filenames listed in the indexglob passed in."""
 	myindexlen=len(myindex)
 	startpos=0
@@ -176,7 +191,10 @@ def getitem(myid,myitem):
 def xpand(myid,mydest):
 	myindex=myid[0]
 	mydata=myid[1]
-	origdir=os.getcwd()
+	try:
+		origdir=os.getcwd()
+	except:
+		origdir="/"
 	os.chdir(mydest)
 	myindexlen=len(myindex)
 	startpos=0
@@ -246,47 +264,47 @@ class tbz2:
 		This function is called by relevant functions already."""
 		try:
 			mystat=os.stat(self.file)
+			if self.filestat:
+				changed=0
+				for x in [ST_SIZE, ST_MTIME, ST_CTIME]:
+					if mystat[x] != self.filestat[x]:
+						changed=1
+				if not changed:
+					return 1
+			self.filestat=mystat
+			a=open(self.file,"r")
+			a.seek(-16,2)
+			trailer=a.read()
+			self.infosize=0
+			self.xpaksize=0
+			if trailer[-4:]!="STOP":
+				a.close()
+				return 0
+			if trailer[0:8]!="XPAKSTOP":
+				a.close()
+				return 0
+			self.infosize=decodeint(trailer[8:12])
+			self.xpaksize=self.infosize+8
+			a.seek(-(self.xpaksize),2)
+			header=a.read(16)
+			if header[0:8]!="XPAKPACK":
+				a.close()
+				return 0
+			self.indexsize=decodeint(header[8:12])
+			self.datasize=decodeint(header[12:16])
+			self.indexpos=a.tell()
+			self.index=a.read(self.indexsize)
+			self.datapos=a.tell()
+			a.close()
+			return 2
 		except:
 			return 0
-		if self.filestat:
-			changed=0
-			for x in [ST_SIZE, ST_MTIME, ST_CTIME]:
-				if mystat[x] != self.filestat[x]:
-					changed=1
-			if not changed:
-				return 1
-		self.filestat=mystat
-		a=open(self.file,"r")
-		a.seek(-16,2)
-		trailer=a.read()
-		self.infosize=0
-		self.xpaksize=0
-		if trailer[-4:]!="STOP":
-			a.close()
-			return 0
-		if trailer[0:8]!="XPAKSTOP":
-			a.close()
-			return 0
-		self.infosize=decodeint(trailer[8:12])
-		self.xpaksize=self.infosize+8
-		a.seek(-(self.xpaksize),2)
-		header=a.read(16)
-		if header[0:8]!="XPAKPACK":
-			a.close()
-			return 0
-		self.indexsize=decodeint(header[8:12])
-		self.datasize=decodeint(header[12:16])
-		self.indexpos=a.tell()
-		self.index=a.read(self.indexsize)
-		self.datapos=a.tell()
-		a.close()
-		return 2
 
 	def filelist(self):
 		"""Return an array of each file listed in the index."""
 		if not self.scan():
 			return None
-		return getindex(self.index)
+		return getindex_mem(self.index)
 
 	def getfile(self,myfile,mydefault=None):
 		"""Finds 'myfile' in the data segment and returns it."""
@@ -312,7 +330,10 @@ class tbz2:
 		"""Unpacks all the files from the dataSegment into 'mydest'."""
 		if not self.scan():
 			return 0
-		origdir=os.getcwd()
+		try:
+			origdir=os.getcwd()
+		except:
+			origdir="/"
 		a=open(self.file,"r")
 		if not os.path.exists(mydest):
 			os.makedirs(mydest)
